#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;
uniform float time;
uniform bool animateWave;

void animateVertexWave() {
	vec3 transformedPosition = vec3(position);
	float frequency = 0.7f;
	float amplitude = 0.2f; 
	float angleX = (time + position.x) * frequency;
	float angleZ = (time + position.z) * frequency;
	float totalAngle = (angleX + angleZ) / 2;
	transformedPosition.y += sin(totalAngle) * amplitude;

	// recalculate normal
	vec3 newNormal = normalize(vec3(
		-amplitude * frequency * cos(totalAngle), 
		1.0f, 
		-amplitude * frequency * cos(totalAngle)));
	vert.normal_world = normalMatrix * newNormal;

	vec4 position_world_ = modelMatrix * vec4(transformedPosition, 1);
	vert.position_world = position_world_.xyz;
	gl_Position = viewProjMatrix * position_world_;
}

void main() {
	vert.uv = uv;
	if(animateWave) {
		animateVertexWave();
	} else {
		vert.normal_world = normalMatrix * normal;
		vec4 position_world_ = modelMatrix * vec4(position, 1);
		vert.position_world = position_world_.xyz;
		gl_Position = viewProjMatrix * position_world_;
	}
}